<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 52f0f1294a70 $
 * $Author: lukasimhof $
 * $Date: 2022-12-19 18:24 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_backup_preparer{

	private static function checkFilePermission(): bool{
		if(!is_writable(TEMP_PATH)){
			we_backup_util::addLog('Error: Can\'t write to ' . TEMP_PATH);
			t_e('Error: Can\'t write to ' . TEMP_PATH);
			return false;
		}
		return true;
	}

	private static function prepare(): bool{
		if(!self::checkFilePermission()){
			return false;
		}
		we_backup_settings::setSessionData();
		return true;
	}

	public static function prepareExport(): bool{
		we_update_updater::fixInconsistentTables();
		if(!self::prepare()){
			return false;
		}
		$settings = we_backup_settings::i();
		$settings->class = new we_backup_exportZIP(we_backup_util::FILE_PREFIX . we_base_request::_(we_base_request::FILE, 'filename'));

		if($settings->option(we_backup_settings::OPTION_BACKUP_EXTERN)){
			$settings->extern_files = self::getFileList();
			$settings->extern_files_count = count($settings->extern_files);
		}

		$recs = DB_WE::$inst->getAllq('SHOW TABLE STATUS LIKE "' . addTblPrefix('tbl') . '%"');
		if(!$recs){
			throw new we_exception_database('No Tables found!');
		}
		foreach($recs as $cur){
			$name = we_backup_util::getDefaultTableName($cur['Name']);
			if($name !== ''){
				$settings->row_count += ((int) $cur['Rows']);
				$settings->limits['avgLen'][$name] = $cur['Avg_row_length'];
			}
		}
		if(!$settings->row_count){
			throw new we_exception_database('No Tables found!');
		}

		return true;
	}

	public static function prepareImport(): bool{
		if(!self::prepare()){
			return false;
		}

		try{
			$settings = we_backup_settings::i();
			$settings->backup_file = self::getBackupFile();
		} catch (Throwable $e){
			t_e($e->getMessage());
			return false;
		}

		$offset = strlen(we_backup_util::weXmlExImProtectCode);
		$line = we_base_file::loadLine($settings->backup_file, 0, ($offset));
		if(empty($line)){
			return false;
		}
		$settings->offset = ($line == we_backup_util::weXmlExImProtectCode) ? $offset : 0;
		$settings->format = we_backup_util::getFormat($settings->backup_file, $settings->offset);

		switch($settings->format){
			case we_backup_exportZIP::EXPORT_TYPE:
				$settings->class = $class = new we_backup_importZIP();
				try{
					$info = $class->getInfo();
				} catch (we_exception_file $e){
					t_e('file format not supported');
					return false;
				}
				$settings->weVersion = $info->WE;
				$settings->offset_end = $info->BackupFiles + $info->BinFiles;
				break;
			default:
				t_e('file format not supported');
				return false;
		}
		if($settings->isHandled(we_modules_info::CORE)){
			self::clearTemporaryData(FILE_TABLE);
			$settings->files_to_delete = self::getFileLists();
			$settings->files_to_delete_count = count($settings->files_to_delete);
		}

		if($settings->isHandled(we_backup_settings::OPTION_BACKUP_VERSIONS) || $settings->isHandled(we_modules_info::CORE) || $settings->isHandled(we_modules_info::OBJECT) || $settings->isHandled(we_backup_settings::OPTION_BACKUP_VERSION_BIN)
		){
			self::clearVersionData();
		}

		if($settings->isHandled(we_modules_info::OBJECT)){
			self::clearTemporaryData(OBJECT_FILES_TABLE);
		}

		return true;
	}

	/**
	 * @return string
	 * @throws we_exception_file
	 */
	private static function getBackupFile(): string{
		if(($backup_select = we_base_request::_(we_base_request::FILE, 'backup_select'))){
			return WE_BACKUP_PATH . we_backup_util::SAVE_DIR . $backup_select;
		}

		$fileUploader = new we_fileupload_resp_base();
		$commitedFile = $fileUploader->commitUploadedFile(we_fileupload::REPLACE_BY_FILENAME, WE_BACKUP_DIR . we_backup_util::SAVE_DIR, true);
		we_base_file::insertIntoCleanUp($commitedFile, 0);
		return $commitedFile;
	}

	/**
	 *
	 * @return string[]
	 */
	private static function getFileLists(): array{
		return
			array_merge(
			self::getFileList(TEMPLATES_PATH, true, false),
										 self::getSiteFiles());
	}

	/**
	 *
	 * @param string $dir
	 * @param bool $with_dirs
	 * @param bool $rem_doc_root
	 * @return string[]
	 */
	public static function getFileList(string $dir = '', bool $with_dirs = false, bool $rem_doc_root = true): array{
		$dir = (string) ($dir ?: WE_DOCUMENT_ROOT);
		if(!is_readable($dir) || !is_dir($dir)){
			throw new we_exception_file('Dir not readable ' . $dir);
		}
		$thumbDir = trim(WE_THUMBNAIL_DIRECTORY, '/');
		$list = [];
		$d = dir($dir);
		while(false !== ($entry = $d->read())){
			switch($entry){
				case '.':
				case '..':
				case 'webEdition':
				case 'dummy':
				case $thumbDir:
					continue 2;
				default:
					$file = $dir . '/' . $entry;
					if(!self::isPathExist(str_replace(WE_DOCUMENT_ROOT, '', $file))){
						if(is_dir($file)){
							if($with_dirs){
								$list[] = self::addToFileList($file, $rem_doc_root);
							}
							$list = array_merge($list, self::getFileList($file, $with_dirs, $rem_doc_root));
						} else {
							$list[] = self::addToFileList($file, $rem_doc_root);
						}
					} elseif(is_dir($file)){
						$list = array_merge($list, self::getFileList($file, $with_dirs, $rem_doc_root));
					}
			}
		}
		$d->close();
		return $list;
	}

	/**
	 * @param string $file
	 * @param bool $rem_doc_root
	 * @return string
	 */
	private static function addToFileList(string $file, bool $rem_doc_root = true): string{
		return ($rem_doc_root ? str_replace(WE_DOCUMENT_ROOT, '', $file) : $file);
	}

	/**
	 *
	 * @global type $DB_WE
	 * @return string[]
	 */
	private static function getSiteFiles(): array{
		$out = [];
		foreach(self::getFileList(WE_DOCUMENT_ROOT . WE_SITE_DIR, true, false) as $file){
			//don't use f/getHash since RAM usage
			switch(fS('SELECT ContentType FROM ' . FILE_TABLE . ' WHERE Path="' . we_database_base::$inst->escape(str_replace(WE_DOCUMENT_ROOT . rtrim(WE_SITE_DIR, '/'), '', $file)) . '"')){
				case we_base_ContentTypes::IMAGE:
				case we_base_ContentTypes::APPLICATION:
				case we_base_ContentTypes::PDF:
				case we_base_ContentTypes::VIDEO:
				case we_base_ContentTypes::AUDIO:
					continue 2;
				default:
					$out[] = $file;
			}
		}
		return $out;
	}

	private static function clearTemporaryData(string $docTable): void{
		$DB_WE = we_database_base::$inst;
		switch($docTable){
			case FILE_TABLE:
				$DB_WE->query('DELETE FROM ' . CONTENT_TABLE . ' WHERE DocumentTable="' . TEMPORARY_DOCTARGET . '"');
				break;
			case OBJECT_FILES_TABLE:
				$tableIDs = $DB_WE->getAllq('SELECT ID FROM ' . OBJECT_TABLE . ' WHERE IsFolder=0', true);
				foreach($tableIDs as $id){
					$DB_WE->query('DELETE FROM ' . OBJECT_X_TABLE . $id . ' WHERE OF_DocumentTable="' . TEMPORARY_DOCTARGET . '"');
				}
		}
		$DB_WE->query('DELETE FROM ' . HISTORY_TABLE . ' WHERE DocumentTable="' . stripTblPrefix($docTable) . '"');
		$DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID' . ($docTable === FILE_TABLE ? '=0' : '>0'));
	}

	private static function clearVersionData(): void{
		DB_WE::$inst->query('TRUNCATE TABLE ' . VERSIONS_TABLE . ';');
		$path = WE_VERSION_PATH;
		if(($dir = opendir($path))){
			while(($file = readdir($dir))){
				switch($file){
					case '.':
					case '..':
						break;
					default:
						if(!is_dir($file)){
							we_base_file::deleteLocalFile($path . $file);
						}
				}
			}
			closedir($dir);
		}
	}

	private static function isPathExist(string $path): bool{
		return ((fB('SELECT 1 FROM ' . FILE_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($path) . '" LIMIT 1')) ||
			(fB('SELECT 1 FROM ' . TEMPLATES_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($path) . '" LIMIT 1')));
	}

	public static function getErrorMessage(): string{
		$settings = we_backup_settings::i();
		if(!($settings->backup_file)){
			$mess = /* (isset($settings->option('upload')) ?
				  sprintf(g_l('backup', '[upload_failed]'), we_base_file::getHumanFileSize(we_fileupload::getMaxUploadSizeB(), we_base_file::SZ_MB)) : */
				g_l('backup', '[file_missing]');
		} else if(!is_readable($settings->backup_file)){
			$mess = g_l('backup', '[file_not_readable]');
		} else if($settings->format !== we_backup_exportZIP::EXPORT_TYPE){
			$mess = g_l('backup', '[format_unknown]');
		} else {
			$mess = g_l('backup', '[unspecified_error]');
		}

		if(!empty($settings->backup_log)){
			we_backup_util::addLog('Error: ' . $mess);
		}

		$cmd = new we_base_jsCmd();
		$cmd->addMsg($mess, we_base_util::WE_MESSAGE_ERROR);
		$cmd->addCmd('location', ['doc' => 'edbody', 'loc' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=recover_backup&pnt=edbody&step=2']);
		return $cmd->getCmds();
	}

}
