<?php

declare(strict_types=1);

/** webEdition CMS
 *
 * $Rev: 771ec18885af $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2023-01-05 15:03 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_base_sessionHandlerCache extends we_base_sessionHandler{

	private const DIRTY_TIME = SYSTEM_WE_SESSION_TIME / 10;
	private const PREFIX_DBTOUCH = 'tch_';
	private const PREFIX_LOCK = 'lck_';
	private const PREFIX_SESSION = 'sess_';

	/** @var int */
	private $lastTouch = 0;

	public function __construct(){
		//echo 'SESS: initCache';
		we_cache_session::setup();
		parent::__construct();
	}

	public function read(/* string */$sessID)/* : string */{
		//echo 'SESS: readFromCache';
		$fullsess = $this->sessionName . $sessID;

		try{
			$max = $this->execTime * 6; //just wait 80% of the time to clear the lock, if not cleared, break the lock
			while(($lockID = we_cache_session::load(self::PREFIX_LOCK . $fullsess)) && (--$max)){
				if($lockID === $this->id){
					//echo 'SESS: lock with OWNID';
					throw new we_exception_cache(); //lock is ours
				}
				//echo 'SESS: locked wait';
				usleep(250000);
			}
			self::$releaseError[] = 'Session ' . $this->sessionName . '/' . $sessID . ' still locked';
			$this->setLock(false, $sessID);
			throw new we_exception_cache(); //finally load data
		}catch(we_exception_cache $ex){
			//no lock found or cleared
			//get last writeback time
			//echo 'SESS: readLastTouch';
			try{
				$this->setLock(true, $sessID);
				$this->lastTouch = (int) we_cache_session::load(self::PREFIX_DBTOUCH . $fullsess);
				//echo 'SESS: readFromCachedData';
				$ret = $this->decryptSessData(we_cache_session::load(self::PREFIX_SESSION . $fullsess), $sessID);
				we_cache_session::touch(self::PREFIX_SESSION . $fullsess);
				we_cache_session::touch(self::PREFIX_DBTOUCH . $fullsess);
				return $ret;
			}catch(we_exception_cache $ex){
				//unable to load data, read data from database backend (cold cache or no session)
				//echo 'SESS: FALLBACK';
			}
		}
		$ret = parent::read($sessID);
		if($ret){//found in db, but not in cache
			$this->write($sessID, $ret, true);
		}
		if(self::$releaseError){
			t_e(self::$releaseError);
		}
		return $ret;
	}

	private function setLock(bool $set, string $sessID): void{
		$fullsess = $this->sessionName . $sessID;
		if($set){
			//echo 'SESS: LOCK';
			we_cache_session::save(self::PREFIX_LOCK . $fullsess, $this->id, $this->execTime);
		} else {
			//echo 'SESS: UNLOCK';
			we_cache_session::clean(self::PREFIX_LOCK . $fullsess);
		}
	}

	public function write($sessID, $sessData, $lock = false): bool{
		if(!$sessID){
			return true;
		}
		if(!$lock && !$sessData){
			self::delSess($sessID, $this->sessionName);
			return true;
		}
		$fullsess = $this->sessionName . $sessID;
		if(md5($sessID . $sessData, true) === $this->hash){
			$ret = $this->updateTimestamp($sessID, '');
			$this->setLock($lock, $sessID);
			if($ret){
				return true;
			}
		}
		we_cache_session::save(self::PREFIX_SESSION . $fullsess, $sessData ? $this->cryptSessData($sessData) : '');
		we_cache_session::save(self::PREFIX_DBTOUCH . $fullsess, ($this->lastTouch = time()));
		$this->setLock($lock, $sessID);
		return parent::write($sessID, $sessData, false); //never lock fallback if cache was written
	}

	protected static function delSess(string $sessID, string $sessName): void{
		$fullsess = $sessName . $sessID;
		we_cache_session::clean(self::PREFIX_SESSION . $fullsess);
		we_cache_session::clean(self::PREFIX_LOCK . $fullsess);
		we_cache_session::clean(self::PREFIX_DBTOUCH . $fullsess);
		parent::delSess($sessID, $sessName);
	}

	public function updateTimestamp($sessID, $val){
		$fullsess = $this->sessionName . $sessID;
		if(!we_cache_session::touch(self::PREFIX_SESSION . $fullsess, self::$sessTime)){//no cache entry
			$this->hash = ''; //enforce write through
			return $this->write($sessID, $val, false);
		}
		we_cache_session::touch(self::PREFIX_DBTOUCH . $fullsess);
		$this->setLock(false, $sessID);
		//only touch if time is more than 30s
		$t = time();
		if((($t - $this->lastTouch) > ((int) self::DIRTY_TIME))){
			we_cache_session::save(self::PREFIX_DBTOUCH . $fullsess, ($this->lastTouch = $t));
			if(!parent::updateTimestamp($sessID, '')){
				return parent::write($sessID, $val, false);
			}
		}
		return true;
	}

}
