<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2846022b1c50 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-06-27 12:58 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_cache_backendRedis implements we_cache_backend{

	/** @var RedisArray */
	private $cache;

	/** @var string */
	private $id = '';

	public function __construct(string $id){
		$this->id = $id;
		$this->setServers(MEMCACHE_SERVERS);
	}

	public function setServers(array $servers): void{
		$this->cache = new RedisArray($servers, [
			'retry_timeout' => 250,
			'consistent' => true,
		]);
		$this->cache->setOption(Redis::OPT_PREFIX, $this->id);
		if(defined('MEMCACHE_BACKEND_DB') && MEMCACHE_BACKEND_DB){
			$this->cache->select(MEMCACHE_BACKEND_DB);
		}
	}

	public function clean(string $hash = ''): bool{
		try{
			return $this->cache->del($hash) > 0;
		}catch(RedisException $e){
			return false;
		}
	}

	/**
	 * @param string $entry
	 * @return mixed
	 * @throws we_exception_cache
	 */
	public function get(string $entry){
		try{
			$data = $this->cache->get($entry);
		}catch(RedisException $e){
			throw new we_exception_cache();
		}
		if($data === false){
			throw new we_exception_cache();
		}
		return $data;
	}

	public function increment(string $entry, int $default = 1, int $expiry = 3600): int{
		try{
			return ($default === 1 && $this->cache->incr($entry) ?
					(int) $this->get($entry) :
					($this->set($entry, $default, $expiry) ?
					$default :
					0)
					);
		}catch(RedisException $e){
			return 0;
		}
	}

	public function set(string $entry, $data, int $expiry): bool{
		try{
			return $this->cache->setEx($entry, $expiry, $data);
		}catch(RedisException $e){
			$this->clean($entry);
			t_e('warning', $entry, strlen(is_string($data) ? $data : serialize($data)), $e);
			return false;
		}
	}

	public function touch(string $entry, int $expiry = 3600): bool{
		try{
			return $this->cache ? $this->cache->expire($entry, $expiry) : false;
		}catch(RedisException $e){
			return false;
		}
	}

	public function quit(): void{
		$this->cache->close();
	}

	public function getStatus(): string{
		$fields = ['Server' => '',
			'Database' => 'db' . MEMCACHE_BACKEND_DB,
			'Uptime (days)' => 'uptime_in_days',
			'Version' => 'redis_version'];

		$hosts = $this->cache->_hosts();
		$ret = new we_html_table(['class' => 'default statusTable'], count($hosts), count($fields));
		$ret->addHeaderRow(array_keys($fields));
		$rowid = 1;
		$colid = 0;
		array_shift($fields);
		foreach($hosts as $host){
			$redis = $this->cache->_instance((string) $host);
			$ret->setCol($rowid, $colid++, (string) $host);
			$info = $redis->info('all');
			foreach($fields as $field){
				$ret->setCol($rowid, $colid++, (string) $info[$field]);
			}
			$rowid++;
		}
		return $ret->getHtml();
	}

}
