<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b73190177417 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-04-30 17:51 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod,PhanUndeclaredExtendedClass
use Sabre\DAV;

class we_davfs_dbDocument extends we_davfs_dbFile{

	protected const HEADER = '/* WE-Header - DO NOT MODIFY
Path: __PATH__
ID:   __ID__
*/
';
	protected const HEADER_HTML = '<!-- WE-Header - DO NOT MODIFY
Path: __PATH__
ID:   __ID__
*-->
';

	function __construct(string $path, string $table, int $id){
		//special handling for scss/less files which are "parsed"
		parent::__construct($path, $table, $id ?: fI('SELECT ID FROM ' . $table . ' WHERE Path REGEXP "^' . DB_WE::$inst->escape(strtr($path, ['.scss' => '\.(css|scss)', '.less' => '\.(css|less)'])) . '$"'));
	}

	public function getHeader(): string{
		$ct = fS('SELECT ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $this->ID);
		if(!$ct){
			return 'no such file';
		}
		switch($ct){
			case we_base_ContentTypes::HTML:
				$header = self::HEADER_HTML;
				break;
			default:
				$header = self::HEADER;
				break;
		}
		return strtr($header, [//@phan-suppress-current-line PhanParamSuspiciousOrder
			'__PATH__' => $this->path,
			'__ID__' => $this->ID,
		]);
	}

	/**
	 *
	 * @return string
	 */
	function get(){
		$ct = fS('SELECT ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $this->ID);
		if(!$ct){
			return 'no such file';
		}
		$doc = we_base_ContentTypes::getObject($ct, $this->ID);
		if(!$doc->userHasPerms()){
			return 'Permission denied';
		}
		/* if($doc->isLockedByUser()){
		  return $this->getLockedDocument($doc);
		  } */
		return $this->getHeader() . $doc->getElementS('data');
	}

	/**
	 * @return string
	 */
	public function getContentType(){
		return 'text/html';
	}

	/**
	 * @param string|resource $data
	 * @return string|null
	 * @throws DAV\Exception\ConflictingLock
	 * @throws DAV\Exception\PreconditionFailed
	 */
	function put($data){
		$content = is_resource($data) ? (stream_get_contents($data) ?: '') : $data;
		if(!$content){
			return null;
		}
		$ct = fS('SELECT ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $this->ID);
		$doc = we_base_ContentTypes::getObject($ct, $this->ID);
		if($doc->isLockedByUser() || !$doc->lockDocument()){
			t_e('unable to lock document ', $this->path, $this->ID);
			throw new DAV\Exception\ConflictingLock();
		}
		$doc->setElementS('data', preg_replace('_(/\*+|<!--)\s*WE-Header[^*>]+(\*+/|\*?-->)\n?_s', '', $content));
		if(!$doc->we_save() || !$doc->we_publish()){
			t_e('unable to write document ', $this->path, $this->ID);
			we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
		return $this->getETag();
	}

	/**
	 * Updates the file based on a range specification.
	 *
	 * The first argument is the data, which is either a readable stream
	 * resource or a string.
	 *
	 * The second argument is the type of update we're doing.
	 * This is either:
	 * * 1. append
	 * * 2. update based on a start byte
	 * * 3. update based on an end byte
	 * ;
	 * The third argument is the start or end byte.
	 *
	 * After a successful put operation, you may choose to return an ETag. The
	 * ETAG must always be surrounded by double-quotes. These quotes must
	 * appear in the actual string you're returning.
	 *
	 * Clients may use the ETag from a PUT request to later on make sure that
	 * when they update the file, the contents haven't changed in the mean
	 * time.
	 *
	 * @param resource|string $data
	 * @param int             $rangeType
	 * @param int             $offset
	 *
	 * @return string|null
	 */
	public function patch($data, $rangeType, $offset = null){
		$doc = new we_document_html($this->ID);
		if($doc->isLockedByUser() || !$doc->lockDocument()){
			throw new DAV\Exception\ConflictingLock();
		}
		$content = $this->doPatch($doc->getElementS('data'), is_resource($data) ? (stream_get_contents($data) ?: '') : $data, $rangeType, $offset ?: 0);

		$doc->setElementS('data', $content);
		if(!$doc->we_publish()){
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
		return $this->getETag();
	}

}
