<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 02c38849404b $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-12-22 12:03 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_dialog_sysinfo{

	public const REQUIRED_EXTENSIONS = [
		'date',
		'filter',
		'mysqli',
		'pcre',
		//'Reflection',
		'session',
		'xml',
		'zlib'
	];

	private static function getInfoTable(array $infoArr): string{
		$table = new we_html_table(['class' => 'sysinfoTable'], 1, 2);

		foreach($infoArr as $k => $v){
			$table->addRow(
					new we_html_tr([
						new we_html_td((string) $k, ['class' => 'bold']),
						new we_html_td(self::parseValue((string) $k, $v))
							])
			);
		}
		return $table->getHtml();
	}

	private static function ini_get_bool(string $val): bool{
		switch(strtolower(ini_get($val) ?: '')){
			case '1':
			case 'on':
			case 'yes':
			case 'true':
				return true;
			default:
				return false;
		}
	}

	private static function ini_get_message(string $val): string{
		$bool = ini_get($val);
		switch($bool === false ? '' : strtolower($bool)){
			case 'on':
			case 'yes':
			case 'true':
				return 'on';
			case 'off':
			case 'no':
			case 'false':
				return 'off';
			case '':
				return g_l('sysinfo', '[not_set]');
			default:
				return (string) $bool;
		}
	}

	private static function parseValue(string $name, /* mixed */ $value): string{
		static $types = [];

		$types = $types ?: [
			'upload_max_filesize' => 'bytes',
			'post_max_size' => 'bytes',
			'memory_limit' => 'bytes',
			'max_allowed_packet' => 'bytes',
			g_l('sysinfo', '[we_max_upload_size]') => 'bytes'
		];

		if(in_array($name, array_keys($types), true)){
			if($types[$name] === 'bytes' && $value){

				$value = we_base_util::convertIniSizes($value);
				return we_base_file::getHumanFileSize($value, we_base_file::SZ_MB) . ' (' . $value . ' Bytes)';
			}
		}

		return $value;
	}

	private static function getConnectionTypes(): array{
		$connectionTypes = [];
		$connectionTypeUsed = '';

		if(ini_get('allow_url_fopen') == 1){
			$connectionTypes[] = 'fopen';
			$connectionTypeUsed = 'fopen';
		}
		if(is_callable('curl_exec')){
			$connectionTypes[] = 'curl';
			if(count($connectionTypes) == 1){
				$connectionTypeUsed = 'curl';
			}
		}
		foreach($connectionTypes as &$con){
			if($con == $connectionTypeUsed){
				$con = '<u>' . $con . '</u>';
			}
		}
		return $connectionTypes;
	}

	private static function getWarning(string $message, string $value): string{
		return we_html_element::div(['class' => 'sysinfoMsg', 'title' => $message], $value . we_html_element::span(['class' => "warn fa-stack fa-lg"], '<i class="fa fa-exclamation-triangle fa-stack-2x" ></i>
  <i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i>')
		);
	}

	private static function getInfo(string $message, string $value): string{
		return we_html_element::div(['class' => 'sysinfoMsg', 'title' => $message], $value . we_html_element::span(['class' => "info fa-stack fa-lg"], '<i class="fa fa-circle fa-stack-2x" ></i>
  <i class="fa fa-info fa-stack-1x fa-inverse"></i>')
		);
	}

	private static function getOK(string $message = '', string $value = ''): string{
		return we_html_element::div(['class' => 'sysinfoMsg', 'title' => $message], $value . we_html_element::span(['class' => "ok fa fa-lg fa-check fa-ok"]));
	}

	public static function checkCrypt(): string{
		if(function_exists('openssl_encrypt') && (array_search('aes-128-gcm', openssl_get_cipher_methods(true), true))){
			$data = 'testWEIsCool';
			$enc = we_customer_customer::cryptData($data);
			$dec = we_customer_customer::decryptData($enc);
			return ($data === $dec ? '' : 'Error: ' . openssl_error_string());
		}
		return 'no crypt';
	}

	public static function getDialog(string $what): void{
		if($what == 'we_gui_phpinfo'){
			phpinfo();
			return;
		}

		$weVersion = WE_VERSION .
				(defined('WE_SVNREV') && WE_SVNREV != '0000' ? ' (Revision: ' . WE_SVNREV . (defined('WE_VERSION_HOTFIX_NR') && WE_VERSION_HOTFIX_NR ? ' , h' . WE_VERSION_HOTFIX_NR . ',' : '') . ((defined('WE_VERSION_BRANCH') && WE_VERSION_BRANCH != 'trunk') ? ' ' . WE_VERSION_BRANCH : '') . ')' : '') .
				(defined('WE_VERSION_SUPP') && WE_VERSION_SUPP ? ' ' . g_l('global', '[' . WE_VERSION_SUPP . ']') : '') .
				(defined('WE_VERSION_SUPP_VERSION') && WE_VERSION_SUPP_VERSION ? WE_VERSION_SUPP_VERSION : '');

		$gdVersion = (defined('GD_VERSION') ? GD_VERSION : '');

		$phpExtensionsDetectable = true;

		$phpextensions = array_map('strtolower', get_loaded_extensions());
		$phpextensionsMissing = [];

		if(count($phpextensions) > 3){
			foreach(self::REQUIRED_EXTENSIONS as $exten){
				if(!in_array(strtolower($exten), $phpextensions, true)){
					$phpextensionsMissing[] = $exten;
				}
			}
		} else {
			$phpExtensionsDetectable = false;
		}

		$sqlVersion = we_database_base::getMysqlVer();
//$allowTempTables = we_search_search::checkRightTempTable();
		$info = [
			'webEdition' => [
				g_l('sysinfo', '[we_version]') => $weVersion,
				g_l('sysinfo', '[server_name]') => $_SERVER['SERVER_NAME'],
				g_l('sysinfo', '[port]') => $_SERVER['SERVER_PORT'] ?: 80,
				g_l('sysinfo', '[protocol]') => we_http_util::getServerProtocol(),
				g_l('sysinfo', '[installation_folder]') => WEBEDITION_PATH,
				/* g_l('sysinfo', '[we_max_upload_size]') => we_fileupload::getMaxUploadSizeB(),// FIXME: handle 0 = noz restricted */
				g_l('import', '[pfx]') => TBL_PREFIX
			],
			'<a href="javascript:showPhpInfo();">PHP</a>' => [g_l('sysinfo', '[php_version]') => PHP_VERSION,
				'max_execution_time' => ini_get('max_execution_time'),
				'memory_limit' => we_base_util::convertIniSizes(ini_get('memory_limit') ?: 0),
				'short_open_tag' => (self::ini_get_bool('short_open_tag')) ? self::getWarning(g_l('sysinfo', '[short_open_tag warning]'), ini_get('short_open_tag') ?: '') : self::ini_get_message('short_open_tag'),
				'allow_url_fopen' => self::ini_get_message('allow_url_fopen'),
				'open_basedir' => self::ini_get_message('open_basedir'),
				'upload_max_filesize' => we_base_util::convertIniSizes(ini_get('upload_max_filesize') ?: ''),
				'post_max_size' => we_base_util::convertIniSizes(ini_get('post_max_size') ?: ''),
				'max_input_vars' => ini_get('max_input_vars') < 2000 ? self::getWarning('<2000', ini_get('max_input_vars') ?: '') : self::getOK('>=2000', self::ini_get_message('max_input_vars')),
				'session.auto_start' => (self::ini_get_bool('session.auto_start')) ? self::getWarning(g_l('sysinfo', '[session.auto_start warning]'), self::ini_get_message('session.auto_start') ?: '') : self::getOK('', self::ini_get_message('session.auto_start')),
				'display_errors' => (self::ini_get_bool('display_errors') ? self::getWarning(g_l('sysinfo', '[display_errors warning]'), 'on') : self::getOK('', self::ini_get_message('off'))),
				'finfo' => (!class_exists('finfo', true) ? self::getWarning(g_l('sysinfo', '[class_missing]'), '') : self::getOK('', '')),
				g_l('sysinfo', '[umlautdomains]') => (!(function_exists('idn_to_ascii') && defined('IDNA_DEFAULT') && defined('INTL_IDNA_VARIANT_UTS46')) ? self::getWarning(g_l('sysinfo', '[umlautdomains_warning]'), '') : self::getOK('', '')),
			],
			'Database' => [
				'Typ' => $sqlVersion[1],
				'Version'/* g_l('sysinfo', '[mysql_version]') */ => (version_compare('5.6.0', $sqlVersion[0]) > 1) ? self::getWarning(sprintf(g_l('sysinfo', '[dbversion warning]'), $sqlVersion[0]), $sqlVersion[0]) : self::getOK('', $sqlVersion[0]),
				'max_allowed_packet' => (string) DB_WE::$inst->getMaxAllowedPacket(),
				'lock tables' => (DB_WE::$inst->hasLock() ? self::getOK('', g_l('sysinfo', '[available]')) : self::getWarning('', '-')),
				'native driver (mysqlnd)' => (defined('MYSQLI_OPT_INT_AND_FLOAT_NATIVE') ? self::getOK('', g_l('sysinfo', '[available]')) : self::getWarning('mysqlnd not used - this is a security issue', '-')),
				//'create temporary tables' => ($allowTempTables ? self::getOK('', g_l('sysinfo', '[available]')) : self::getWarning('', '-')),
				'Info' => DB_WE::$inst->getInfo(),
			],
			'System' => [g_l('sysinfo', '[connection_types]') => implode(', ', self::getConnectionTypes()),
				g_l('sysinfo', '[mbstring]') => (is_callable('mb_get_info') ? g_l('sysinfo', '[available]') : '-'),
				g_l('sysinfo', '[gdlib]') => ($gdVersion ? g_l('sysinfo', '[version]') . ' ' . $gdVersion : '-'),
				g_l('sysinfo', '[exif]') => (is_callable('exif_imagetype') ? g_l('sysinfo', '[available]') : self::getWarning(g_l('sysinfo', '[exif warning]'), '-')),
				g_l('sysinfo', '[pcre]') => ((defined('PCRE_VERSION')) ? ( g_l('sysinfo', '[version]') . ' ' . PCRE_VERSION ) : self::getWarning(g_l('sysinfo', '[available]'), g_l('sysinfo', '[pcre_unkown]'))),
				g_l('sysinfo', '[phpext]') => (!empty($phpextensionsMissing) ? self::getWarning(g_l('sysinfo', '[phpext warning2]'), g_l('sysinfo', '[phpext warning]') . implode(', ', $phpextensionsMissing)) : g_l('sysinfo', ($phpExtensionsDetectable ? '[available]' : '[detectable warning]')) ),
				g_l('sysinfo', '[crypt]') => (!($tmp = self::checkCrypt()) ? self::getOK() : self::getWarning(g_l('sysinfo', '[crypt_warning]'), $tmp)),
				'openssl' => defined('OPENSSL_VERSION_TEXT') ? OPENSSL_VERSION_TEXT : g_l('sysinfo', '[phpext warning]'),
			],
		];

		$parts = new we_html_multiIconBoxContent();
		foreach($info as $k => $v){
			$parts->add((new we_html_multiIconBoxContentLine($k, self::getInfoTable($v)))->space(we_html_multiIconBox::SPACE_MED2));
		}
		$parts
				->add((new we_html_multiIconBoxContentLine(g_l('cache', '[cache]'), we_cache_we::getStatus()))->space(we_html_multiIconBox::SPACE_MED2));

		$buttons = we_html_button::formatButtons(we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();"));

		echo we_html_tools::getHtmlTop(g_l('sysinfo', '[sysinfo]'), we_html_element::jsScript(JS_DIR . 'sysinfo.js'), we_html_element::body([
					'class' => 'weDialogBody', 'onload' => "self.focus();"
						], '<div id="info" style="display: block;">' .
								(new we_html_multiIconBox($parts))
								->marginLeft()
								->buttons(we_html_button::create_button(we_html_button::NEXT, "javascript:showPhpInfo();") . $buttons)
								->get() . '
	</div>
	<div id="more" style="display:none;">' .
								(new we_html_multiIconBox((new we_html_multiIconBoxContent())
										->add((new we_html_multiIconBoxContentLine('', '<iframe id="phpinfo" style="width:100%;height:500px;"></iframe>')))
								))
								->marginLeft()
								->buttons(we_html_button::create_button(we_html_button::BACK, "javascript:showInfoTable();") . $buttons)
								->get() . '
	</div>'
		));
	}

}
