<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 87b15f573da1 $
 * $Author: Lukas Imhof $
 * $Date: 2021-12-06 11:08 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * this class implements the cache functionality for the glossary
 *
 */
class we_glossary_cache{
	/**
	 * language of the cache
	 *
	 * @var string
	 */
	var $language = '';

	/**
	 * internal id of the cache
	 *
	 * @var array
	 */
	private $content = [];

	/**
	 * Constructor
	 *
	 * @param string $language
	 */
	function __construct(string $language){
		$this->language = $language;
	}

	/**
	 * deletes the cache file
	 *
	 * @return boolean
	 */
	function clear(): bool{
		we_cache_file::clean(we_cache_file::DOMAIN_GLOSSAR . $this->language);
		return true;
	}

	/**
	 * write the given entries into the cache file
	 *
	 * @return array
	 */
	function write(): array{
		$DB_WE = DB_WE::$inst;

		$all = $DB_WE->getAllq('SELECT Text,Type,Language,Title,Attributes,LENGTH(Text) as Length,Fullword FROM ' . GLOSSARY_TABLE . ' WHERE Language="' . $DB_WE->escape($this->language) . '" AND IsPublished=1 ORDER BY Length DESC');
		$Items = [];

		foreach($all as $rec){
			$Type = $rec['Type'];
			$Attributes = array_map('trim', we_unserialize($rec['Attributes']));
			$Text = htmlspecialchars(trim($rec['Text']), ENT_NOQUOTES);
			$Title = htmlspecialchars(trim($rec['Title']), ENT_QUOTES);

			$temp = ['Fullword' => $rec['Fullword']];

			if($Title){
				$temp['title'] = $Title;
			}

			// Language
			if(!empty($Attributes['lang'])){
				$temp['lang'] = $Attributes['lang'];
				$temp['xml:lang'] = $temp['lang'];
			}

			// Language
			if($Type == we_glossary_glossary::TYPE_LINK){
				$temp['class'] = 'glossarLink';
				$urladd = [];

				if(isset($Attributes['mode'])){
					switch($Attributes['mode']){
						// External Link
						case we_glossary_glossary::MODE_LINK_EXT:
							// Href
							$temp['href'] = (!empty($Attributes['ExternUrl']) && $Attributes['ExternUrl'] != we_base_link::EMPTY_EXT ? $Attributes['ExternUrl'] : '');

							// Parameter
							if(!empty($Attributes['ExternParameter'])){
								$urladd = array_merge($urladd, explode(' ', $Attributes['ExternParameter']));
							}
							break;
						// Internal Link
						case we_glossary_glossary::MODE_LINK_INT:
							// LinkID
							if(!empty($Attributes['InternLinkID'])){
								$tmp = id2path((int) $Attributes['InternLinkID']);
								$path_parts = pathinfo($tmp);
								$temp['href'] = (seoIndexHide($path_parts['basename']) ?
									($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
									$tmp);
							} else {
								$temp['href'] = '';
							}

							// Parameter
							if(!empty($Attributes['InternParameter'])){
								$urladd = array_merge($urladd, explode(' ', $Attributes['InternParameter']));
							}
							break;
						// Object Link
						case we_glossary_glossary::MODE_LINK_OBJ:
							// LinkID
							$temp['href'] = (empty($Attributes['ObjectLinkPath']) ? '' : $Attributes['ObjectLinkPath']);

							if(!empty($Attributes['ObjectLinkID'])){
								$urladd[] = WEOBJ_PARAM . '=' . $Attributes['ObjectLinkID'];
							}

							// Parameter
							if(!empty($Attributes['ObjectParameter'])){
								$urladd = array_merge($urladd, explode(' ', $Attributes['ObjectParameter']));
							}
							break;
						case we_glossary_glossary::MODE_LINK_CAT:// Category Link
							switch($Attributes['modeCategory'] ?? we_glossary_glossary::MODE_CATEGORY_EXT){
								case we_glossary_glossary::MODE_CATEGORY_INT:
									// LinkID
									if(!empty($Attributes['CategoryInternLinkID'])){
										$tmp = id2path((int) $Attributes['CategoryInternLinkID']);
										$path_parts = pathinfo($tmp);
										$temp['href'] = (seoIndexHide($path_parts['basename']) ?
											($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
											$tmp);
									} else {
										$temp['href'] = '';
									}
									break;
								case we_glossary_glossary::MODE_CATEGORY_EXT:
								default:
									// Href
									$temp['href'] = (empty($Attributes['CategoryUrl']) ? '' : $Attributes['CategoryUrl']);
							}

							// Cat Parameter & Cat ID
							if(!empty($Attributes['CategoryCatParameter']) && !empty($Attributes['CategoryLinkID'])){
								$urladd[] = $Attributes['CategoryCatParameter'] . '=' . $Attributes['CategoryLinkID'];
							}

							// Parameter
							if(!empty($Attributes['CategoryParameter'])){
								$urladd = array_merge($urladd, explode(' ', $Attributes['CategoryParameter']));
							}
							break;
					}
				}


				// Attribute
				if(!empty($Attributes['attribute'])){
					$temp['attribute'] = ' ' . $Attributes['attribute'] . ' ';
				}
				foreach(['target', 'hreflang', 'accesskey', 'tabindex', 'rel'] as $attr){
					if(!empty($Attributes[$attr])){
						$temp[$attr] = $Attributes[$attr];
					}
				}

				$temp['href'] .= str_replace(['?&', '&&'], ['?', '&'], ($urladd ? '?' . implode('&', $urladd) : '')) .
					// Anchor
					(!empty($Attributes['anchor']) ?
					'#' . $Attributes['anchor'] :
					'');
			}

			$Items[$Text][$Type] = $temp;
		}

		$content = [we_glossary_glossary::TYPE_LINK => [],
			we_glossary_glossary::TYPE_ACRONYM => [],
			we_glossary_glossary::TYPE_ABBREVATION => [],
			we_glossary_glossary::TYPE_FOREIGNWORD => [],
			we_glossary_glossary::TYPE_TEXTREPLACE => [],
		];

		foreach($Items as $Text => $Value){
			$prefix = $postfix = $Tag = '';
			foreach($Value as $Type => $AttributeList){

				switch($Type){
					case we_glossary_glossary::TYPE_LINK:
						$Tag = 'a';
						break;
					case we_glossary_glossary::TYPE_ACRONYM:
						$Tag = 'acronym';
						break;
					case we_glossary_glossary::TYPE_ABBREVATION:
						$Tag = 'abbr';
						break;
					case we_glossary_glossary::TYPE_FOREIGNWORD:
						$Tag = 'span';
						break;
					case we_glossary_glossary::TYPE_TEXTREPLACE:
						$Tag = '';
						break;
				}

				if($Tag != ''){
					$prefix .= '<' . $Tag;
				}
				$full = $AttributeList['Fullword'];
				unset($AttributeList['Fullword']);

				if($Type != we_glossary_glossary::TYPE_TEXTREPLACE){
					foreach($AttributeList as $Attribute => $Val){
						$prefix .= ($Attribute === 'attribute' ? $Val : ' ' . $Attribute . '="' . $Val . '"');
					}
				} else {
					$prefix .= $AttributeList['title'];
				}
				if($Tag != ''){
					$prefix .= '>';
					$postfix = '</' . $Tag . '>' . $postfix;
				}

				//make sure we found a whole word!
				if($full){//no quotes - they're used for attributes
					$content[$Type]['-(^|(?![^<>]*/|[^<>]*>)\s|(?![^<>]*/|[^<>]*>)[!#$%&\'()*+,\-./:;=?@[\\]^_`{\|}~])(' . preg_quote($Text, '-') . ')(\s|[!"#$%&\'()*+,\-./:;=?@[\\]^_`{\|}~]|$)-'] = '${1}' . $prefix . ($Tag ? '${2}' : '') . $postfix . '${3}';
				} else {
					$content[$Type]['-((?![^<>]*/|[^<>]*>)[^<>]+)(' . preg_quote($Text, '-') . ')-'] = '${1}' . $prefix . ($Tag ? '${2}' : '') . $postfix;
					//$content[$Type]['-' . preg_quote($Text, '-') . '-'] = $prefix . ($Tag ? '${0}' : '') . $postfix;
				}
			}
		}
		we_cache_file::save(we_cache_file::DOMAIN_GLOSSAR . $this->language, $content, 3600, SERIALIZE_PHP);
		return $content;
	}

	/**
	 * get all entries from the cache
	 *
	 * @return array
	 */
	function get(string $type): array{
		if(!$this->content){
			try{
				$this->content = we_cache_file::load(we_cache_file::DOMAIN_GLOSSAR . $this->language);
			} catch (we_exception_cache $e){
				$this->content = self::write();
			}
		}
		return ($this->content && !empty($this->content[$type]) ?
			$this->content[$type] :
			[]);
	}

}
